/**
 * Die Buchstabenfolge
 *
 *      CCCCCBAAAACCCCA
 *
 * stellt eine wichtige geheime Nachricht dar.
 * Das Beigefügte Programm erlaubt es nach Fertigstellung diese Folge Zeichen für Zeichen in einer Zeichen-Queue zu speichern (Methode befuelle).
 *
 * In Java funktionieren Queue folgendermassen:
 *
 * heraus<-- 'c''c'...'b''a'<--hinein
 *          Kopf            Ende
 *
 * Das Programm enthält zwei Versuche eine solche Zeichen-Queue in kompakter Form abzuspeichern.
 * Das ist einerseits die Methode transform, welche die Zeiche-Queue in eine Queue mit Folgenden Objekten umwandelnt:
 *
 * 'C''b''a''c''a'
 *  5  1  4  4  1
 *
 * Dabei steht jedes Objekt für einen Block aufeinanderfolgender gleicher Buchstaben. Die Länge des Blocks wird im Objekt abgespeichert.
 *
 * Ein weiterer Versuch einer Speicherung wird durch die Methode trasnsform_to_map realisiert. Hier wird die Zeichen-Queue durch eine Map
 * abgespeicher. Dabei dient die Nummer der Startposition jedes neuen Blocks gleicher Buchstaben als Schlüssel und der Buchstabe des Bocks selbst
 * als Wert.
 *
 * Vervollständigen Sie das beigefügte Programm an den Punkten a, b, c und d. Bei richtiger Lösung liefert das Programm die unten angegebene
 * Ausgabe
 *
 * Vergessen Sie nicht folgende Zusatzfragen:
 *
 * Welchen Nachteil der Datenstruktur Queue offenbart das Programm? (Hinweis: Denken Sie an die Aufrufe der Methode befuelle in main)
 * Wäre es sinnvoll, als Alternative die Map entweder
 * a) mit Buchstaben eines Blocks als Schlüssel und der Anzahl der Buchstabenwiederholungen als Wert oder
 * b) mit beide Dinge vertauscht zu realisieren?
 *
 *
 * Ausgabe des fertigen Programms
 *
 * Die Methode befuelle wird ausgeführt
 * c,c,c,c,c,b,a,a,a,a,c,c,c,c,a
 *
 * Die Methode transform wird ausgeführt
 * Ausgabe der Methode drucke_spezialqueue:
 * c 5
 * b 1
 * a 4
 * c 4
 * a 1
 *
 * Die Methode befuelle wird ausgeführt
 * c,c,c,c,c,b,a,a,a,a,c,c,c,c,a
 * Die Methode transform_to_map wird ausgeführt.
 * Ausgabe der Methode drucke_spezialmap:
 * 1 c
 * 6 b
 * 7 a
 * 11 c
 * 15 a
 * 16
 *
 *
 *
 */


import java.util.Queue;
import java.util.LinkedList;  // Für Queue

import java.util.Map;
import java.util.TreeMap;  // Wie in der Vorlesung vom 23.11.


public class AdHoc7 {
    Character buchstabe;
    int anz_wiederholungen;

    public AdHoc7(Character buchst, int anz_wied) {
        /*********************** Teil a ********************************/
        // Erstellen Sie den Rumpf des Konstruktors, sodass
        // die beiden Objektvariablen sinnvoll initialisiert werden.
    }

    public static void befuelle(Queue<Character> character_queue_local) {
        System.out.println();
        System.out.println("Die Methode befuelle wird ausgefuehrt.");
        System.out.println();

        for (int i = 1; i <= 5; i++) {
            character_queue_local.offer('C');
        }

        character_queue_local.offer('B');

        for (int i = 1; i <= 4; i++) {
            character_queue_local.offer('A');
        }

        /************************ Teil b **********************************/
        // Fügen Sie auch das Ende der Nachricht "CCCCA" Zeichen für Zeichen
        // in die Queue character_queue_local ein.
        // Sie können dazu die Methode offer verwenden.
    }

    public static Queue<AdHoc7> transform(Queue<Character> character_queue_local) {
        System.out.println();
        System.out.println("Die Methode transform wird ausgefuehrt.");
        System.out.println();

        Queue<AdHoc7> result_queue = new LinkedList<AdHoc7>();

        Character single_char = character_queue_local.poll();
        int char_counter = 1;

        while (single_char != null) {
            Character single_char_next = character_queue_local.poll();

            while (single_char_next == single_char) {
                char_counter++;
                single_char_next = character_queue_local.poll();
            }

            // System.out.println(""+ single_char + " " + char_counter);

            /*********************** Teil c ********************************/
            // Erzeugen Sie ein Objekt vom Typ geheime_nachricht welches
            // die Werte single_char und char_counter in den
            // Objektvariablen enthält.
            // Fügen Sie dieses Objekt in die Queue result_queue ein.
            // Sie können dazu die Methode offer verwenden.

            single_char = single_char_next;
            char_counter = 1;
        }

        return result_queue;
    }

    public static void drucke_spezialqueue(Queue<AdHoc7> spezialqueue) {
        System.out.println("Ausgabe der Methode drucke_spezialqueue:");
        AdHoc7 single_obj = spezialqueue.poll();

        while (single_obj != null) {
            System.out.println(single_obj.buchstabe + " " + single_obj.anz_wiederholungen);
            single_obj = spezialqueue.poll();
        }
    }

    public static Map<Integer, Character> transform_to_map(Queue<Character> character_queue_local) {
        System.out.println();
        System.out.println("Die Methode transform_to_map wird ausgefuehrt.");
        System.out.println();

        Map<Integer, Character> result_map = new TreeMap<Integer, Character>();  // Wie in der Vorlesung vom 23.11.

        Character single_char = character_queue_local.poll();
        int char_counter = 1;
        int start_position = 1;

        while (single_char != null) {
            Character single_char_next = character_queue_local.poll();

            while (single_char_next == single_char) {
                char_counter++;
                single_char_next = character_queue_local.poll();
            }

            // System.out.println(""+ single_char + " " + char_counter + " " + start_position);

            /*********************** Teil d ********************************/
            // Fügen Sie die errechnete Startposition start_position eines
            // Buchstabenblocks als Key und den Buchstaben single_char
            // als Wert zur Map result_map dazu.
            // Sie können dazu die Methode put verwenden.

            single_char = single_char_next;
            start_position += char_counter;
            char_counter = 1;
        }

        result_map.put(start_position, ' ');

        return result_map;
    }

    public static void drucke_spezialmap(Map<Integer, Character> spezialmap, int max_key) {
        System.out.println("Ausgabe der Methode drucke_spezialmap:");

        for (int k = 1; k <= max_key; k++) {
            Character ein_buchstabe = spezialmap.get(k);

            if (ein_buchstabe != null) {
                System.out.println(k + " " + ein_buchstabe);
            }

        }
    }

    public static void main(String[] args) {
        Queue<Character> character_queue = new LinkedList<Character>();

        befuelle(character_queue);

        System.out.println(character_queue);
        // Ausgabe: [C, C, C, C, C, B, A, A, A, A, C, C, C, C, A]

        Queue<AdHoc7> obj_queue = transform(character_queue);

        drucke_spezialqueue(obj_queue);

        // **************************************************************************************************

        befuelle(character_queue);

        System.out.println(character_queue);
        // Ausgabe: [C, C, C, C, C, B, A, A, A, A, C, C, C, C, A]

        Map<Integer, Character> bloecke = transform_to_map(character_queue);

        drucke_spezialmap(bloecke, 1000);
        // 1000 ist die maximale Länge der Nachricht.
    }
}
