/**
 * Der Code muss objektorientierter umgeschrieben werden und der Punkt soll auf den KeyListener reagieren (sprich die Richtung ändern)
 */
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.Timer;
import java.awt.event.*;

@SuppressWarnings("serial")
public class AdHoc6 extends JPanel implements ActionListener, KeyListener {
    int x = 0;
    int y = 0;
    Timer timer;

    public AdHoc6()
    {
        addKeyListener(this);
        timer = new Timer(1000, this);
        timer.start();
        setFocusable(true);
        requestFocusInWindow();
    }

    private void moveBall() {
        x = x + 30;
        y = y;
    }
    @Override
    public void paint(Graphics g) {
        super.paint(g);
        Graphics2D g2d = (Graphics2D) g;
        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        g2d.fillOval(x, y, 30, 30);
    }

    @Override
    public void keyTyped(KeyEvent e){}

    @Override
    public void keyPressed(KeyEvent e){
        switch (e.getKeyCode()){
            case KeyEvent.VK_LEFT:
                System.out.println("left");
                break;
            case KeyEvent.VK_RIGHT:
                System.out.println("right");
                break;
            case KeyEvent.VK_UP:
                System.out.println("up");
                break;
            case KeyEvent.VK_DOWN:
                System.out.println("down");
                break;
        }
    }

    @Override
    public void keyReleased(KeyEvent e){}


    public void actionPerformed(ActionEvent e)
    {
        if (e.getSource() == timer)
        {
            this.moveBall();
            this.repaint();
        }
    }

    public static void main(String[] args) throws InterruptedException {
        JFrame frame = new JFrame("Sample Frame");
        AdHoc6 game = new AdHoc6();
        frame.add(game);
        frame.setSize(300, 400);
        frame.setVisible(true);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }
}


// mögliche Lösung

/*
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.Timer;
import java.awt.event.*;
import java.util.ArrayList;

@SuppressWarnings("serial")
public class Game extends JPanel implements ActionListener, KeyListener {
    Point point;
    Point direction;
    Timer timer;
    final int speed = 30;
    public static int winSizeX = 300;
    public static int winSizeY = 400;

    public Game()
    {
        point = new Point(0, 0);
        direction = new Point(0, 0);
        addKeyListener(this);
        timer = new Timer(100, this);
        timer.start();
        setFocusable(true);
        requestFocusInWindow();
    }

    private void moveBall() {
        Point p = point.getLast();
        point.addPoint(new Point(p.getX() + (direction.getX() * speed), p.getY() + (direction.getY() * speed)));
    }
    @Override
    public void paint(Graphics g) {
        super.paint(g);
        Graphics2D g2d = (Graphics2D) g;
        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        ArrayList<Point> points = point.getList();
        System.out.println(points.size());
        int sizeX = 30, sizeY = 30;
        for(Point p : points) {
            g2d.fillOval(p.getX(), p.getY(), sizeX, sizeY);
            sizeX -= 3;
            sizeY -= 3;
        }

    }

    @Override
    public void keyTyped(KeyEvent e){}

    @Override
    public void keyPressed(KeyEvent e){
        switch (e.getKeyCode()){
            case KeyEvent.VK_LEFT:
                System.out.println("left");
                direction.setX(-1);
                direction.setY(0);
                break;
            case KeyEvent.VK_RIGHT:
                System.out.println("right");
                direction.setX(1);
                direction.setY(0);
                break;
            case KeyEvent.VK_UP:
                System.out.println("up");
                direction.setX(0);
                direction.setY(-1);
                break;
            case KeyEvent.VK_DOWN:
                System.out.println("down");
                direction.setX(0);
                direction.setY(1);
                break;
        }
    }

    @Override
    public void keyReleased(KeyEvent e){}


    public void actionPerformed(ActionEvent e)
    {
        if (e.getSource() == timer)
        {
            if(direction.getX() != 0 || direction.getY() != 0) {
                this.moveBall();
                point = point.cutoffLast();
                this.repaint();
            }
        }
    }

    public static void main(String[] args) throws InterruptedException {
        JFrame frame = new JFrame("Sample Frame");
        Game game = new Game();
        frame.add(game);
        frame.setSize(Game.winSizeX, Game.winSizeY);
        frame.setVisible(true);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }
}

/////////////////////////////////////////////////////////////////////////////////////////////////////
import java.util.ArrayList;


public class Point {
    private int x;
    private int y;
    private final int length = 8;

    private Point nextPoint;

    public Point(int x, int y) {
        if(x >= Game.winSizeX) {
            this.x = x % Game.winSizeX;
        } else if(x < 0) {
            this.x = Game.winSizeX + x;
        } else {
            this.x = x;
        }
        if(y >= Game.winSizeY) {
            this.y = y % Game.winSizeY;
        } else if(y < 0){
            this.y = Game.winSizeY + y - 30;
        } else {
            this.y = y;
        }
    }

    public ArrayList<Point> getList() {
        if(nextPoint == null) {
            ArrayList<Point> thisPoint = new ArrayList<>();
            thisPoint.add(this);
            return thisPoint;
        } else {
            ArrayList<Point> al = nextPoint.getList();
            al.add(this);
            return al;
        }
    }

    public Point cutoffLast() {
        if(nextPoint != null) {
            int i = nextPoint.getIndex();
            if(i >= length) {
                return nextPoint;
            }
        }
        return this;
    }

    private int getIndex() {
        if(nextPoint == null) {
            return 0;
        } else {
            return 1 + nextPoint.getIndex();
        }
    }

    public void addPoint(Point p) {
        if(nextPoint == null) {
            nextPoint = p;
        } else {
            nextPoint.addPoint(p);
        }
    }

    public Point getLast() {
        if(nextPoint == null) {
            return this;
        } else {
            return nextPoint.getLast();
        }
    }

    public void moveBy(int x, int y) {
        this.x += x;
        this.y += y;
    }

    public void setX(int newX) {
        x = newX;
    }

    public void setY(int newY) {
        y = newY;
    }

    public int getX() {
        return x;
    }

    public int getY() {
        return y;
    }
}



 */
